/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Map-Service
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-09-10, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * -------------------------------------------------------------------
  * TODO:
  *       - implement it
  */
package jaugment2.map;

  //JDK
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.MarshalledObject;
import java.awt.geom.Point2D;
import java.awt.Dimension;
  //JINI
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.UnknownLeaseException;
import net.jini.core.lease.Lease;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;


/**
 * Title:        GPS-service-interface
 * Description:  Monitors wireless-lan link-quality
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Marcus Wolschon Suran@gmx.net
 * @version 1.0
 */

public interface GPSProxyIF extends Remote
{
 public static class GPSPosition
 {
  private double lat;
  private double lon;

  double getLatitude()
  {
   return(lat);
  }
  double getLongtitude()
  {
   return(lon);
  }
  public GPSPosition(double lat, double lon)
  {
   this.lat = lat;
   this.lon = lon;
  }

  public String toString()
  {
   return("[latitude:"+lat+",longtitude:"+lon+"]");
  }
 }


 public abstract GPSPosition getCurrentPosition() throws RemoteException;

  /** Subscribe for an GPSEvent
    */
 public Lease addListener(RemoteEventListener listener, MarshalledObject handback, int registerfor) throws RemoteException, LeaseDeniedException, IllegalStateException;


 /**
   * Events thrown by a GPS-service.
   * bearing is for proxys that also implement CompassService
   */
 public static class GPSServiceEvent extends RemoteEvent
 {
  public static final int EVID_GPSPOSITIONUPDATED = 1;
  public static final int EVID_COMPASSBEARINGUPDATED = 2;
  private GPSPosition pos;
  private double bearing;
  public GPSPosition getCurrentPosition()
  {return(pos);}
  public double getBearing()
  {return(bearing);}

  public GPSServiceEvent(Object proxy, long eventID, long seqNum, MarshalledObject handback, GPSPosition pos)
  {
   super(proxy, eventID, seqNum, handback);
   this.pos = pos;
  }


 }


}
