/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Map-Service
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-09-08, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  *       - drawing the map works
  * 0.0.2 (2001-09-09, Marcus Wolschon<Suran@gmx.net>)
  *       - correct part of the map is shown
  *       - reacts to user-input
  * 0.0.3 (2001-10-01, Marcus Wolschon<Suran@gmx.net>)
  *       - loads selected maps
  *       - loads all map-images
  *       - all StackTraces go to Logger.err
  * 0.0.4 (2001-12-06, Marcus Wolschon<Suran@gmx.net>)
  *       - can zoom
  * -------------------------------------------------------------------
  * TODO:
  *       - blend multiple maps
  *       - allow scrolling in the map
  */
package jaugment2.map;

   //JDK
import javax.swing.*;
import java.awt.*;
     //JINI
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.UnknownLeaseException;
import net.jini.core.lease.Lease;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;
   //Project
import jaugment2.helpers.Logger;
import java.awt.event.*;


/**
 * Title:        jAugment - Map
 * Description:  A simple map with GPS-support planned
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Marcus Wolschon Suran@gmx.net
 * @version 1.0
 */

public class MapPanel extends JComponent implements RemoteEventListener
{
 private class myMapPanel  extends JPanel
 {
  public void paint(Graphics g)
   {
    try
      {
       if(currentPosition != null)
       if(currentMapImages != null)
       if(currentMapImages.length > 0) //TODO: for all maps, support overlaying
         {
          int imgheight = currentMapImages[0].getHeight(
                                                        new java.awt.image.ImageObserver()
                                                          {
                                                       public boolean imageUpdate(Image img, int infoflags,
                                			       int x, int y, int width, int height)
                                                             {
                                                              return(true);
                                                             }
                                                          }

                                                       );
          int imgwidth =  currentMapImages[0].getWidth(
                                                        new java.awt.image.ImageObserver()
                                                          {
                                                       public boolean imageUpdate(Image img, int infoflags,
                                			       int x, int y, int width, int height)
                                                             {
                                                              return(true);
                                                             }
                                                          }

                                                       );


          java.awt.geom.Point2D pos = currentPosition.interpolate(new java.awt.Dimension(imgheight, imgwidth),
                                                                  currentMaps[0].getLeftupper(),
                                                                  currentMaps[0].getRightlower()
                                                                 );


          int srcX1 = Math.max((int)(pos.getX()-(getWidth()*zoom_factor/2)),0);
          int srcY1 = Math.max((int)(pos.getY()-(getHeight()*zoom_factor/2)),0);
          int srcX2 = Math.min((int)(pos.getX()+(getWidth()*zoom_factor/2)),imgwidth);
          int srcY2 = Math.min((int)(pos.getY()+(getHeight()*zoom_factor/2)),imgheight);
          Logger.debug.println("DEBUG[myMapPanel]: srcX1="+srcX1+" srcX2="+srcX2+" srcY1="+srcY1+" srcY2="+srcY2+" imgheight="+imgheight+" imgwidth="+imgwidth+" getHeight="+getHeight()+" getWidth="+getWidth()+" currentPosition="+currentPosition+" currentMaps[0].getLeftupper()="+currentMaps[0].getLeftupper()+", currentMaps[0].getRightlower()="+currentMaps[0].getRightlower()+"");
          g.drawImage(currentMapImages[0],
	              0,0,getWidth(), getHeight(),
                      srcX1,
                      srcY1,
                      srcX2,
                      srcY2,
                      java.awt.Color.white,
                      new java.awt.image.ImageObserver()
                          {
                       public boolean imageUpdate(Image img, int infoflags,
			       int x, int y, int width, int height)
                             {
                              return(true);
                             }
                          }
                     );
          return;
         }
      }
    catch(Exception e)
      {
       e.printStackTrace(Logger.err);
      }
    super.paint(g);

   }


   public float zoom_factor = 1;
 }



 private MapServiceProxyIF proxy = null;
  myMapPanel jPanelMap = new myMapPanel();
  JPanel jPanelRight = new JPanel();
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel jPanelMaps = new JPanel();
  JList jListMaps = new JList();
  JButton jButtonSelectMaps = new JButton();
  BorderLayout borderLayout2 = new BorderLayout();
  JPanel jPanelPos = new JPanel();
  JTextField jTextFieldPosition = new JTextField();
  BorderLayout borderLayout3 = new BorderLayout();
  JButton jButtonSetPosition = new JButton();
  BorderLayout borderLayout4 = new BorderLayout();
  JCheckBox jCheckBoxGPS = new JCheckBox();


  private MapServiceProxyIF.Map currentMaps[];
  private MapServiceProxyIF.Position currentPosition;
  private java.awt.Image currentMapImages[] = new java.awt.Image[0];
  JPanel jPanel_zoom_buttons = new JPanel();
  JButton jButton_zoom_in = new JButton();
  JButton jButton_zoom_out = new JButton();

  public MapPanel(MapServiceProxyIF proxy)
  {
   this.proxy = proxy;
    try
    {
     currentPosition = proxy.getCurrentPosition();
     currentMaps = proxy.getMapsForPosition(currentPosition);
      jbInit();

        // fill in position in GUI
     jTextFieldPosition.setText(currentPosition.getX()+","+currentPosition.getY());

        // filll in maps in GUI

     DefaultListModel dlm = new DefaultListModel();
     if(currentMaps != null)
     for(int i=0; i < currentMaps.length; i++)
        dlm.addElement(currentMaps[i]);
     jListMaps.setModel(dlm);

     if(currentMaps != null)
     if(currentMaps.length > 0)
       {
            //TODO: only for selected maps, others later
        currentMapImages = new java.awt.Image[currentMaps.length];
        for(int i=0; i<currentMaps.length; i++)
          currentMapImages[i] = proxy.getMap(currentMaps[i].getID());

       }


       // register for position-iupdates
      proxy.addListener(this, null, MapServiceProxyIF.MapServiceEvent.EVID_POSITIONUPDATED);


    }
    catch(Exception e)
    {
      e.printStackTrace(Logger.err);
    }

  }

  public MapPanel()
  {
    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace(Logger.err);
    }
  }
  private void jbInit() throws Exception
  {
    this.setLayout(borderLayout1);
    jButtonSelectMaps.setText("select");
    jButtonSelectMaps.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        jButtonSelectMaps_actionPerformed(e);
      }
    });
    jPanelMaps.setLayout(borderLayout2);
    jTextFieldPosition.setText("0,0");
    jPanelPos.setLayout(borderLayout3);
    jButtonSetPosition.setText("set");
    jButtonSetPosition.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        jButtonSetPosition_actionPerformed(e);
      }
    });
    jPanelRight.setLayout(borderLayout4);
    jCheckBoxGPS.setEnabled(false);
    jCheckBoxGPS.setText("GPS");
    jListMaps.setToolTipText("select the maps to show");
    jPanelMap.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mouseClicked(MouseEvent e)
      {
        jPanelMap_mouseClicked(e);
      }
    });
    jButton_zoom_in.setMargin(new Insets(2, 2, 2, 2));
    jButton_zoom_in.setText("+");
    jButton_zoom_in.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        jButton_zoom_in_actionPerformed(e);
      }
    });
    jButton_zoom_out.setToolTipText("");
    jButton_zoom_out.setMargin(new Insets(2, 2, 2, 2));
    jButton_zoom_out.setText("-");
    jButton_zoom_out.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        jButton_zoom_out_actionPerformed(e);
      }
    });
    this.add(jPanelMap, BorderLayout.CENTER);
    this.add(jPanelRight, BorderLayout.EAST);
    jPanelRight.add(jPanelMaps, BorderLayout.CENTER);
    jPanelMaps.add(jButtonSelectMaps,  BorderLayout.SOUTH);
    jPanelMaps.add(jListMaps,  BorderLayout.CENTER);
    jPanelRight.add(jPanelPos,  BorderLayout.SOUTH);
    jPanelPos.add(jTextFieldPosition, BorderLayout.CENTER);
    jPanelPos.add(jButtonSetPosition, BorderLayout.WEST);
    jPanelPos.add(jCheckBoxGPS,  BorderLayout.NORTH);
    jPanelPos.add(jPanel_zoom_buttons,  BorderLayout.SOUTH);
    jPanel_zoom_buttons.add(jButton_zoom_in, null);
    jPanel_zoom_buttons.add(jButton_zoom_out, null);
  }


  /**
   * set new position to what the user has said
   */
  void jButtonSetPosition_actionPerformed(ActionEvent evt)
  {
   try
     {
      String s =  jTextFieldPosition.getText();
      int kommapos = s.indexOf(',');
      if(kommapos == -1)
       {
        jTextFieldPosition.setText(currentPosition.getX()+","+currentPosition.getY());
        return;
       }
      double x = Double.parseDouble(s.substring(0,kommapos));
      double y = Double.parseDouble(s.substring(kommapos+1));
      proxy.setCurrentPosition(new MapServiceProxyIF.Position(x,y));
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }
  }


  /**
   * react to a remote-event
   */
  public void notify(RemoteEvent p0) throws UnknownEventException//, RemoteException
  {
   if(!(p0 instanceof MapServiceProxyIF.MapServiceEvent))
     throw(new UnknownEventException("expecting MapServiceEvent") );

    MapServiceProxyIF.MapServiceEvent mevt = (MapServiceProxyIF.MapServiceEvent)p0;

    currentPosition = mevt.getCurrentPosition();


       // fill in our current position in GUI
    jTextFieldPosition.setText(currentPosition.getX()+","+currentPosition.getY());

   if(currentMaps != null)
   if(currentMaps.length > 0)
   if(!currentMaps[0].isIn(currentPosition)) // we have maps but we wandered off their edges
   try
     {
      currentMaps = proxy.getMapsForPosition(currentPosition);


         // fill in maps in GUI

      DefaultListModel dlm = new DefaultListModel();
      if(currentMaps != null)
      for(int i=0; i < currentMaps.length; i++)
         dlm.addElement(currentMaps[i]);
      jListMaps.setModel(dlm);

    //TODO: can we still use the maps or get new ones
     if(currentMaps != null)
     if(currentMaps.length > 0)
       {
         // get the images for all maps
            //TODO: only for selected maps, others later
        currentMapImages = new java.awt.Image[currentMaps.length];
        for(int i=0; i<currentMaps.length; i++)
          currentMapImages[i] = proxy.getMap(currentMaps[i].getID());
       }
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }
   this.repaint();

  }

  void jPanelMap_mouseClicked(MouseEvent evt)
  {
   try
     {
      //TODO: wander arround in the map
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }
  }

  void jButtonSelectMaps_actionPerformed(ActionEvent e)
  {
   // get selected maps
   Object selectedmaps[] = jListMaps.getSelectedValues();

   //TODO: keep maps that we have downloaded
   currentMapImages = new java.awt.Image[selectedmaps.length];
   for(int i=0; i<selectedmaps.length; i++)
      try{currentMapImages[i] = proxy.getMap(((MapServiceProxyIF.Map)selectedmaps[i]).getID());}catch(java.rmi.RemoteException ex) {ex.printStackTrace(Logger.err);}
  }

  void jButton_zoom_in_actionPerformed(ActionEvent e)
  {
   this.jPanelMap.zoom_factor = this.jPanelMap.zoom_factor / 2;
   this.jPanelMap.repaint(10);
  }

  void jButton_zoom_out_actionPerformed(ActionEvent e)
  {
   this.jPanelMap.zoom_factor = this.jPanelMap.zoom_factor * 2;
   this.jPanelMap.repaint(10);
  }


}
