/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Map-Service
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-09-08, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  *       - it works as a demo
  * 0.0.2 (2001-09-09, Marcus Wolschon<Suran@gmx.net>)
  *       - setting/getting the position works
  * 0.0.3 (2001-09-17, Marcus Wolschon<Suran@gmx.net>)
  *       - maps are configurable
  *       - tries to find GPS-services on the net
  * -------------------------------------------------------------------
  * TODO:
  *       - implement configuring the maps
  *       - draw an icon for this service
  *       - search for GPS and compass-services
  *       - implement waypoints and routes
  *       - implement loading EPS-charts
  *       - add NMEA and Garmin-binary -code (lying around but license-problem)
  */
package jaugment2.map;


  //JDK
//import java.rmi.server.UnicastRemoteObject;
import java.rmi.RemoteException;
import java.rmi.MarshalledObject;
import java.util.LinkedList;
import java.util.Hashtable;
import java.io.FileInputStream;
  //JINI
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.UnknownLeaseException;
import net.jini.core.lease.Lease;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;
import jaugment2.jini.LookupEventListener;
import jaugment2.jini.LookupEventEvent;
import net.jini.lease.LeaseRenewalManager;
  //serviceui-draft-standart
import net.jini.lookup.entry.UIDescriptor;
  //project
import jaugment2.jini.simpleService.VerySimpleService;
import jaugment2.helpers.Logger;
import jaugment2.jini.SimpleLandlord;
import jaugment2.jini.SimpleLandlord.LeaseListener;
import jaugment2.jini.SimpleLandlord.EventRegistration;

//helper-mark for configure to determine that rmic needs to be executed for this file
//rmimakehelper



/**
 * Title:        jAugment - Map
 * Description:  A simple map with GPS-support planned
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Marcus Wolschon Suran@gmx.net
 * @version 1.0
 */

public class MapService extends VerySimpleService implements MapServiceProxyIF, LookupEventListener
{
 private static final boolean debugThis = true;
 private LeaseRenewalManager lrm = new LeaseRenewalManager();

  public MapService() throws java.rmi.RemoteException , java.io.IOException
  {
   if(debugThis)Logger.debug.println("DEBUG: MapService starting up...");
   landLord = new SimpleLandlord(simpleLeaselistener);
   init(this, jaugment2.map.MapServiceProxyIF.class);


   //TODO: make this configurable
   try
     {
      if(System.getProperties().get("jaugment2.map.MapService.maps") != null)
        {
         if(System.getProperties().get("jaugment2.map.MapService.maps") instanceof java.util.LinkedList)
           {
            java.util.LinkedList conf_maps = (java.util.LinkedList)System.getProperties().get("jaugment2.map.MapService.maps");
            for(int i=0; i<conf_maps.size(); i++)
               {
                if(conf_maps.get(i)==null)
                   {
                    Logger.err.println("ERROR: MapService: illegal configuration: system-property map "+i+" is empty - ignoring it");
                    continue;
                   }
               if(!(conf_maps.get(i) instanceof java.util.LinkedList))
                   {
                    Logger.err.println("ERROR: MapService: illegal configuration: system-property map "+i+" is not a linked-list - ignoring it");
                    continue;
                   }
                java.util.LinkedList conf_map = (java.util.LinkedList)conf_maps.get(i);
                if(conf_map.size() != 6)
                   {
                    Logger.err.println("ERROR: MapService: illegal configuration: system-property map "+i+" has not 6 entries - ignoring it");
                    continue;
                   }
                try
                  {
                   maps.add(
                         new MapServiceProxyIF.Map(conf_map.get(0).toString(),
                                   new MapServiceProxyIF.Position(Double.parseDouble(conf_map.get(1).toString()),Double.parseDouble(conf_map.get(2).toString())),
                                   new MapServiceProxyIF.Position(Double.parseDouble(conf_map.get(3).toString()),Double.parseDouble(conf_map.get(4).toString())),
                                   new java.net.URL(conf_map.get(5).toString())
                                  )

                          );
                  }
                catch(Exception e)
                  {
                   e.printStackTrace(Logger.err);
                  }

               }
           }
         else
           {
            Logger.err.println("ERROR[MapService]: illegal configuration: system-property jaugment2.map.MapService.maps defines not a linked-list of maps");
           }
        }
      else
        {
         Logger.warning.println("WARNING: MapService: no maps defined. Please set the system-property jaugment2.map.MapService.maps");
         if(System.getProperty("test")==null)
            Logger.warning.println("WARNING: MapService: in fact no config-ption seems to be visible");
        }
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }
   currentPos = new MapServiceProxyIF.Position(500,500);
/*   maps.add(
         new MapServiceProxyIF.Map("test",
                                   new MapServiceProxyIF.Position(0,0),
                                   new MapServiceProxyIF.Position(2894,2352),
                                   new java.net.URL("file:/home/download/COMBINED_mqmapgend2_w5_6.gif")
                                  )

           );*/

  }


  public String getComment()
  {return("A simple map with GPS-support planned");};
  public String getCreator()
  {return("Marcus Wolschon <Suran@gmx.net>");};
  public UIDescriptor getUIDescriptor()
  {
   if(debugThis)Logger.debug.println("DEBUG: MapService.getUIDescriptor() called...");
   return(MapServiceSwingRepresentation.getUIDescriptor());
  };





 MapServiceProxyIF.Position currentPos = new MapServiceProxyIF.Position(500,500);


 public MapServiceProxyIF.Position getCurrentPosition() throws RemoteException
 {
  return(currentPos);
 }

 public void setCurrentPosition(MapServiceProxyIF.Position p) throws RemoteException
 {
  if(p==null)
    {
     Logger.err.println("ERROR[MapService]: setCurrentPosition was given a null!");
     return;
    }
  currentPos = p;
  fireEvent(MapServiceProxyIF.MapServiceEvent.EVID_POSITIONUPDATED);
 }


 private LinkedList maps = new LinkedList();


 public MapServiceProxyIF.Map[] getMapsForPosition(MapServiceProxyIF.Position p) throws RemoteException
 {
  try
    {
     LinkedList result = new LinkedList();
     for(int i=0; i < maps.size(); i++)
       if(((MapServiceProxyIF.Map)maps.get(i)).isIn(p))
          result.add(maps.get(i));
     return((MapServiceProxyIF.Map[])result.toArray(new MapServiceProxyIF.Map[0]));
     }
  catch(Exception e)
    {
     e.printStackTrace(Logger.err);
    }
  return(new MapServiceProxyIF.Map[0]);

 }



 private java.awt.Image getImage(java.net.URL url) throws Exception
 {
  java.awt.Image img = null;
  java.awt.image.ImageProducer ip =  (java.awt.image.ImageProducer) url.getContent();
  if(ip == null)
    {img = null;;}
  java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
    {img=tk.createImage(ip);}
  return(img);
 }


 public java.awt.Image getMap(Object id) throws java.rmi.RemoteException
 {
  try
    {
     return(getImage((java.net.URL)id));
    }
  catch(Exception e)
    {
     e.printStackTrace();
     return(null);
    }
 }











   private LinkedList listenerRegistrations[] = new LinkedList[]{new LinkedList(),new LinkedList()};

  /**
   * Helper-class for our Landlord.
   */
  private LeaseListener simpleLeaselistener = new LeaseListener()
  {
   public void leaseCanceled(int type, Object reg, int newcount)
   {
//    EventRegistration reg2 = (EventRegistration)reg;
    //TODO
   };
  };

  private SimpleLandlord landLord;

  private void fireEvent(int type)
  {
   if(debugThis)Logger.debug.println("DEBUG: [MapService:fireEvent] I have "+listenerRegistrations[type].size()+" registrations");
   for(int i=0; i<listenerRegistrations[type].size(); i++)
     {
      try
        {                     //TODO: do this asyncronously
         EventRegistration entry = (EventRegistration)listenerRegistrations[type].get(i);
         if(debugThis)Logger.debug.println("DEBUG: [MapService:fireEvent] delivering event type="+type+" subscribedFor="+entry.subscribedFor+" entry.subscribedFor & type="+(entry.subscribedFor & type));
         if( (entry.subscribedFor & type) != 0)
         entry.listener.notify(
                               new MapServiceProxyIF.MapServiceEvent(getProxy(),
                                                     type,
                                                     entry.seq++,
                                                     entry.handback,
                                                     currentPos
                                                    )
                              );
        }
      catch(RemoteException ex)
        {
         Logger.err.println("ERROR: [MapService:fireEvent] could not deliver event");
         ex.printStackTrace(Logger.err);
        }
      catch(UnknownEventException ex)
        {
         Logger.err.println("ERROR: [MapService:fireEvent] could not deliver event - the receiver does not know it");
         ex.printStackTrace(Logger.err);
        }
     }
  };

  /** Subscribe for events
    */
  public Lease addListener(RemoteEventListener listener, MarshalledObject handback, int registerfor) throws RemoteException, LeaseDeniedException, IllegalStateException
  {
   if(listener==null)
     return(null);

   try
     {
      EventRegistration reg = new EventRegistration(listener, handback);
      Lease   l = landLord.grantLease(0, reg, 2*60*10);

      if(l != null)
        {
          // tokenize the expression
         if(debugThis)Logger.debug.println("DEBUG: [MapService] addListener expr=\""+registerfor+"\"");

         for(int i=1; i<(2*registerfor) && (i<listenerRegistrations.length); i=i*2)
            if((i & registerfor) == i)
             {
              listenerRegistrations[i].add(reg);
             };
        };

      return(l);
     }
   catch(Exception ex)
     {
      ex.printStackTrace(Logger.err);
     }
   return(null);
  };







//---------------------------------------------

  private Hashtable serviceID2GPSService = new Hashtable();
  MapGPSListener myGPSListener = null;

  public void serviceFoundEvent(LookupEventEvent e)
  {
   if(debugThis)Logger.debug.println("DEBUG: [MapService] serviceFoundEvent");
   if(e.item != null)
   if(e.item.service != null)
     {
      try
        {
          // integrate rememberance-agents
         if(e.item.service instanceof GPSProxyIF)
           {
            if(debugThis)Logger.debug.println("DEBUG: [MapService] found a GPSProxyIF");
            GPSProxyIF gpspi = (GPSProxyIF)e.item.service;

            //TODO: become a listener of it

            if(myGPSListener ==null)
              myGPSListener = new MapGPSListener(this);
            Lease l = gpspi.addListener(myGPSListener, null, GPSProxyIF.GPSServiceEvent.EVID_GPSPOSITIONUPDATED);
            if(l != null)
              lrm.renewFor(l, Lease.FOREVER, null);
            serviceID2GPSService.put(e.item.serviceID, gpspi);
           }
        }
      catch(Exception e2)
        {
         e2.printStackTrace(Logger.err);
        }

     }

  }

  public void serviceGoneEvent(LookupEventEvent e)
  {
   if(debugThis)Logger.debug.println("DEBUG: [MapService] serviceGoneEvent");
   if(e.item != null)
   if(e.item.service != null)
     {
      if(e.item.service instanceof GPSProxyIF)
        {
         serviceID2GPSService.remove(e.item.serviceID);
        }

     }

  }

  public void serviceChangedEvent(LookupEventEvent e)
  {
   if(debugThis)Logger.debug.println("DEBUG: [MapService] serviceChangedEvent");
  }

}
