/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Map-Service
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-09-08, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2001-09-09, Marcus Wolschon<Suran@gmx.net>)
  *       - events
  *       - setPosition
  *       - positions are implemented
  * -------------------------------------------------------------------
  * TODO:
  *       - implement GPS-coordiates
  */
package jaugment2.map;

  //JDK
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.MarshalledObject;
import java.awt.geom.Point2D;
import java.awt.Dimension;
  //JINI
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.UnknownLeaseException;
import net.jini.core.lease.Lease;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;


/**
 * Title:        WLan-Monitor
 * Description:  Monitors wireless-lan link-quality
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Marcus Wolschon Suran@gmx.net
 * @version 1.0
 */

public interface MapServiceProxyIF extends Remote
{
 public static class Position
 {
  private double x;
  private double y;

  double getX()
  {
   return(x);
  }
  double getY()
  {
   return(y);
  }
  public Position(double x, double y)
  {
   this.x = x;
   this.y = y;
  }

  public Point2D interpolate(Dimension d, Position leftupper, Position rightlower)
  {
   return(new Point2D.Double(
                              d.getWidth() *((x-leftupper.getX())/(rightlower.getX()-leftupper.getX())),
                              d.getHeight()*((y-leftupper.getY())/(rightlower.getY()-leftupper.getY()))
                             )
         );
  }
  //TODO: GPS-soords
  public String toString()
  {
   return("["+x+","+y+"]");
  }
 }

 public static class Map
 {
  private String name;
  public String getName()
  {return(name);}

  private Object ID;
  public Object getID()
  {return(ID);}

  private Position leftupper;
  public Position getLeftupper()
  {return(leftupper);}

  private Position rightlower;
  public Position getRightlower()
  {return(rightlower);}

  public boolean isIn(Position p)
  {
   return( !( rightlower.getX() < p.getX())
            &&
           !( rightlower.getY() < p.getY())
            &&
           !( p.getX() < leftupper.getX())
            &&
           !( p.getY() < leftupper.getY())
         );
  };


  public Map(String name, Position leftupper, Position rightlower, Object ID)
  {
   this.name = name;
   this.leftupper = leftupper;
   this.rightlower = rightlower;
   this.ID = ID;
  }




  public String toString()
  {return(name);}
 }


 public abstract MapServiceProxyIF.Position getCurrentPosition() throws RemoteException;
 public abstract void setCurrentPosition(Position p) throws RemoteException;
 public abstract MapServiceProxyIF.Map[] getMapsForPosition(MapServiceProxyIF.Position p) throws RemoteException;

  /** Subscribe for event
    */
 public Lease addListener(RemoteEventListener listener, MarshalledObject handback, int registerfor) throws RemoteException, LeaseDeniedException, IllegalStateException;


 public static class MapServiceEvent extends RemoteEvent
 {
  public static final int EVID_POSITIONUPDATED = 1;
  private Position pos;
  public Position getCurrentPosition()
  {return(pos);}

  public MapServiceEvent(Object proxy, long eventID, long seqNum, MarshalledObject handback, Position pos)
  {
   super(proxy, eventID, seqNum, handback);
   this.pos = pos;
  }


 }


 public java.awt.Image getMap(Object ID) throws java.rmi.RemoteException;

}