/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Map-Service
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-09-12, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * -------------------------------------------------------------------
  * TODO:
  *       - test it
  */
package jaugment2.map;

  //JDK
//import java.rmi.server.UnicastRemoteObject;
import java.rmi.RemoteException;
import java.rmi.MarshalledObject;
import java.util.LinkedList;
import java.io.FileInputStream;
import java.util.*;
import java.io.*;
  //JINI
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.UnknownLeaseException;
import net.jini.core.lease.Lease;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;
  //serviceui-draft-standart
import net.jini.lookup.entry.UIDescriptor;
  //project
import jaugment2.jini.simpleService.VerySimpleService;
import jaugment2.helpers.Logger;
import jaugment2.jini.SimpleLandlord;
import jaugment2.jini.SimpleLandlord.LeaseListener;
import jaugment2.jini.SimpleLandlord.EventRegistration;

//helper-mark for configure to determine that rmic needs to be executed for this file
//rmimakehelper


/** an implementation if GPSProxyIF based on thhe gps-class of Sulawesi,
 *  Copyright 1998 Neill J Newman <njnewm@essex.ac.uk>. It uses a command-
 *  line tool to get the data of a GPS-receiver.
 *
 */
public class SulawesiGPSService extends VerySimpleService  implements GPSProxyIF
{
 private static final boolean debugThis = true;
  private Thread gps =  null;

  public SulawesiGPSService() throws java.rmi.RemoteException , java.io.IOException
  {
   if(debugThis)Logger.debug.println("DEBUG: SulawesiGPSService starting up...");
   landLord = new SimpleLandlord(simpleLeaselistener);
   init(this, jaugment2.map.GPSProxyIF.class);


   //TODO: make this configurable
   currentPos = new GPSProxyIF.GPSPosition(0,0);
   gps = new gps(this);
   gps.setDaemon(true);
   gps.start();
  }


  public String getComment()
  {return("A GPS-location-service based of the sulawesi-gps-class");};
  public String getCreator()
  {return("Marcus Wolschon <Suran@gmx.net>, original code by Neill J Newman <njnewm@essex.ac.uk>");};
  public UIDescriptor getUIDescriptor()
  {
   if(debugThis)Logger.debug.println("DEBUG: SulawesiGPSService.getUIDescriptor() called...");
   return(null);
  };





 GPSProxyIF.GPSPosition currentPos = new GPSProxyIF.GPSPosition(500,500);


 public GPSProxyIF.GPSPosition getCurrentPosition() throws RemoteException
 {
  return(currentPos);
 }








   private LinkedList listenerRegistrations[] = new LinkedList[]{new LinkedList(),new LinkedList()};

  /**
   * Helper-class for our Landlord.
   */
  private LeaseListener simpleLeaselistener = new LeaseListener()
  {
   public void leaseCanceled(int type, Object reg, int newcount)
   {
//    EventRegistration reg2 = (EventRegistration)reg;
    //TODO
   };
  };

  private SimpleLandlord landLord;

  private void fireEvent(int type)
  {
   if(debugThis)Logger.debug.println("DEBUG: [SulawesiGPSService:fireEvent] I have "+listenerRegistrations[type].size()+" registrations");
   for(int i=0; i<listenerRegistrations[type].size(); i++)
     {
      try
        {                     //TODO: do this asyncronously
         EventRegistration entry = (EventRegistration)listenerRegistrations[type].get(i);
         if(debugThis)Logger.debug.println("DEBUG: [SulawesiGPSService:fireEvent] delivering event type="+type+" subscribedFor="+entry.subscribedFor+" entry.subscribedFor & type="+(entry.subscribedFor & type));
         if( (entry.subscribedFor & type) != 0)
         entry.listener.notify(
                               new GPSProxyIF.GPSServiceEvent(getProxy(),
                                                     type,
                                                     entry.seq++,
                                                     entry.handback,
                                                     currentPos
                                                    )
                              );
        }
      catch(RemoteException ex)
        {
         Logger.err.println("ERROR: [SulawesiGPSService:fireEvent] could not deliver event");
         ex.printStackTrace(Logger.err);
        }
      catch(UnknownEventException ex)
        {
         Logger.err.println("ERROR: [SulawesiGPSService:fireEvent] could not deliver event - the receiver does not know it");
         ex.printStackTrace(Logger.err);
        }
     }
  };

  /** Subscribe for events
    */
  public Lease addListener(RemoteEventListener listener, MarshalledObject handback, int registerfor) throws RemoteException, LeaseDeniedException, IllegalStateException
  {
   if(listener==null)
     return(null);

   try
     {
      EventRegistration reg = new EventRegistration(listener, handback);
      Lease   l = landLord.grantLease(0, reg, 2*60*10);

      if(l != null)
        {
          // tokenize the expression
         if(debugThis)Logger.debug.println("DEBUG: [SulawesiGPSService] addListener expr=\""+registerfor+"\"");

         for(int i=1; i<(2*registerfor) && (i<listenerRegistrations.length); i=i*2)
            if((i & registerfor) == i)
             {
              listenerRegistrations[i].add(reg);
             };
        };

      return(l);
     }
   catch(Exception ex)
     {
      ex.printStackTrace(Logger.err);
     }
   return(null);
  };



//--------------------------------------------------------------------------------

 /**
  *
  * This object listens to the gps input stream and just sends
  * the raw unprocessed output to any requesting classes.
  * This object does not do any processing of the data, it just passes
  * the data on to the classes which requested the information.
  *
  **/

public class gps extends Thread{

  private BufferedReader inStream;
  private PrintWriter outStream;
  private SulawesiGPSService ip;
  private String DataFromGPS = null;
  private double LON = 0.0;
  private double LAT = 0.0;
  private boolean SETUP_OK = false;


  /*
   * the constructor gets called when started from the sulawesi framework
   */
  public gps(SulawesiGPSService creator) {
    // register back with the input decoder.
    ip = creator;

    // setup the gps thingy
    this.setup();

    // starts this object listening for data
    this.start();

  }




  /*
   * set up the interface between the gps executable program (which
   * produces raw gps cordinate data) and this java object.
   */
  private void setup(){


    // load the gps program and get data
    Runtime r = Runtime.getRuntime();
    Process p;
    try{
      // p = r.exec(InputDir + "/gps-pump -P19200  -I -b -e");
      p = r.exec((new java.io.File(".")).getAbsolutePath() + "/pump -P19200 ");

      // connect up the streams to the process
      inStream = new BufferedReader(new InputStreamReader(p.getInputStream()));
      outStream = new PrintWriter(p.getOutputStream());
      SETUP_OK = true;
    }
    catch(IOException error)
    {
      // err....log an error here...or something (copyright beavis and butthead 1998)
     error.printStackTrace(Logger.err);
    }

  }




  /*
   * this method reads the data in from the streams connected to
   * the gps program and sends the data to the objects which have requested it
   */
  public void run(){

      if(SETUP_OK){
	  // ok now this could drag down system resources if a continous
	  // while loop were let loose ;-).. so to try and cut down this object
	  // processing time requirements this loop is sleep every cycle for one
	  // second, which is ok since the gps hardware runs at 10Hz and the program
	  // produces an output once every ten seconds.
	  while (true){
	      try{
		  // read X value
		  DataFromGPS = inStream.readLine();
	      }
	      catch(IOException e){
		  // error in reading fromthe gps proggie.. never mind ;-)
	      }

	      // process the data and send messages
	      processData(DataFromGPS);

	  }
      }
  }



  /*
   * Process the data from the GPS and if a valid result is given
   * then send the position to other objects.
   * Sometimes the GPS program gives, every now and then, checksum errors
   * which is why we need to check if the result is valid.
   */
  private void processData(String inputData){


      // the GPS data comes in the form "GPS LON: 51.87519 LAT: 0.94897"
      // so check that GPS is first, if it is not don;t bother to find anything else
      //try{
      //  if(inputData.startsWith("GPS")){

	      // now get the longitude and latitude
      //	      int index = inputData.indexOf("LON:");
      //      int firstspace = inputData.indexOf(" ", index);
      //      int nextspace = inputData.indexOf(" ", firstspace+1);
      //
      //      LON = inputData.substring(firstspace, nextspace);
      //
      //      index = inputData.indexOf("LAT:");
      //      firstspace = inputData.indexOf(" ", index);
      //      nextspace = inputData.length();
      //
      //      LAT = inputData.substring(firstspace, nextspace);
      //
              // notify objects of the new position
	      // blocks until completed so that the following sleep
	      // call does not sleep the sendNewPosition method.
      //      SendNewPosition(LON, LAT);
      //  }
      //  else{
      //      //System.out.println("invalid data");
      //  }
      //}
      //catch(NullPointerException e){}

      // data is in the format LON = X:Y'Z" LAT = X:Y'Z" where X is degrees,
      // Y is minutes (60th of a degree), and Z is seconds (60th of a minute)
      try{
	  if(inputData.startsWith("LAT")){

	      // now get the longitude and latitude
      	      int index = inputData.indexOf("LAT");
	      int equals = inputData.indexOf("=", index);
	      int nextspace = inputData.indexOf(" ", equals+2);

	      // LAT_DMS = latatude in degrees, minutes, seconds
	      String LAT_DMS = inputData.substring(equals+1, nextspace);

	      // converts latatude DMS to latatude float,
	      // but returns a string of the float ;)
	      LAT = DMS_TO_FLOAT(LAT_DMS);

	      //System.out.println("LAT_DMS = " + LAT_DMS + " LAT = " + LAT);

	      index = inputData.indexOf("LON");
	      equals = inputData.indexOf("=", index);
	      nextspace = inputData.indexOf(" ", equals+2);

	      // LON_DMS = longatude in degrees, minutes, seconds
	      String LON_DMS = inputData.substring(equals+2, nextspace);

	      // converts longatitude DMS to longatitude float,
	      // but returns a string of the float ;)
	      LON = DMS_TO_FLOAT(LON_DMS);

	      //System.out.println("LON_DMS = " + LON_DMS + " LON = " + LON);

              // notify objects of the new position
	      // blocks until completed so that the following sleep
	      // call does not sleep the sendNewPosition method.
             SendNewPosition(LON, LAT);
	  }
	  else{
	      // do nothing cause it wasn't a valid LON/LAT frame
	  }
      }
      catch(NullPointerException e){}

  }


    /*
     * convert LON from degrees, minutes,seconds to floating point, returning
     * the flaot as a string
     */
    private double DMS_TO_FLOAT(String DMS){

	String DMS_AS_FLOAT = null;

	// take in DMS in the form (51:52'32.131104") and convert to a float

	int index = DMS.indexOf(":");
	String SDegrees = DMS.substring(0,index);


	int index2 = DMS.indexOf("\'");
	String  SMinutes = DMS.substring(index+1, index2);
	double Minutes = (Double.parseDouble(SMinutes) / 60) ;



	index = DMS.indexOf("\"");
	String SSeconds = DMS.substring(index2+1, index);
	double Seconds = (Double.parseDouble(SSeconds) / 3600) ;



	// add Minutes and Seconds together
	double Total = Double.parseDouble(SDegrees) + Minutes + Seconds;



	return(Total);

    }




  /*
   * send the position co-ordinates to each object in the notifyhash
   */
  private void SendNewPosition(double LON, double LAT){


    //boolean temp = ip.Manager.sendMessage(this, Message);
   ip.currentPos = new GPSProxyIF.GPSPosition(LAT,LON);
   ip.fireEvent(GPSProxyIF.GPSServiceEvent.EVID_GPSPOSITIONUPDATED);
  }



} // end of gps class





}








